@@ -1,6 +1,11 @@
 #include <stdio.h>
-#include <endian.h>
+#ifndef _WIN32
 #include <assert.h>
+#else
+#define LITTLE_ENDIAN 0
+#define BIG_ENDIAN 1
+#define BYTE_ORDER LITTLE_ENDIAN     
+#endif
 
 #include "general.h"
 #include "vorbismodule.h"
@@ -8,9 +13,9 @@
 #include "pyvorbisinfo.h"
 
 /*  
-   *********************************************************
-                        VorbisFile Object methods 
-   *********************************************************
+*********************************************************
+VorbisFile Object methods 
+*********************************************************
 */
 
 char py_vorbisfile_doc[] = "";
@@ -37,21 +42,21 @@ FDEF(ov_seekable) "Returns whether this VorbisFile is seekable.";
 FDEF(ov_bitrate) "Returns the bitrate of this VorbisFile.";
 
 FDEF(ov_serialnumber) \
-"x.serialnumber(link):\n\n\
+     "x.serialnumber(link):\n\n\
 Returns the serialnumber of this VorbisFile. What's link do?";
 
 FDEF(ov_bitrate_instant) \
-"Returns the bitrate_instant value for this VorbisFile.";
+     "Returns the bitrate_instant value for this VorbisFile.";
 
-FDEF(ov_raw_total) "x.raw_total(i):\n\n\
+     FDEF(ov_raw_total) "x.raw_total(i):\n\n\
 Returns the raw_total value for this VorbisFile. What's i?";
 
 FDEF(ov_pcm_total) \
-"x.pcm_total(i):\n\n\
+     "x.pcm_total(i):\n\n\
 Returns the pcm_total value for this VorbisFile. What's i?";
 
 FDEF(ov_time_total) \
-"x.time_total(i):\n\n\
+     "x.time_total(i):\n\n\
 Returns the time_total value for this VorbisFile. What's i?";
 
 FDEF(ov_raw_seek) "x.raw_seek(pos):\n\nSeeks to raw position pos.";
@@ -65,7 +70,7 @@ FDEF(ov_time_tell) "Returns the time position.";
 FDEF(ov_info) "Return an info object for this file";
 
 FDEF(ov_comment) \
-"Returns a dictionary of lists for the comments in this file.\n\
+     "Returns a dictionary of lists for the comments in this file.\n\
 All values are stored in uppercase, since values should be case-insensitive.";
 
 static PyObject *py_ov_file_getattr(PyObject *, char *name);
@@ -80,7 +85,7 @@ either an open, readable file object or a filename string.\n\
 The most useful method for a VorbisFile object is \"read\".";
 
 PyTypeObject py_vorbisfile_type = {
-  PyObject_HEAD_INIT(&PyType_Type)
+  PyObject_HEAD_INIT(NULL)
   0,
   "VorbisFile",
   sizeof(py_vorbisfile),
@@ -176,6 +181,9 @@ py_ov_file_dealloc(PyObject *self)
   if (PY_VORBISFILE(self))
     ov_clear(PY_VORBISFILE(self));
 
+  /* If file was opened from a file object, decref it, so it can close */
+  Py_XDECREF(((py_vorbisfile *) self)->py_file);
+
   PyMem_DEL(self);
 }
 
@@ -193,7 +201,8 @@ py_ov_open(py_vorbisfile *self, PyObject *args)
   
   if (PyArg_ParseTuple(args, "s|sl", &fname, &initial, &ibytes)) {
 
-    file = fopen(fname, "r");
+    file = fopen(fname, "rb");
+    fobject = NULL;
 
     if (file == NULL) {
       snprintf(errmsg, MSG_SIZE, "Could not open file: %s", fname);
@@ -202,26 +211,50 @@ py_ov_open(py_vorbisfile *self, PyObject *args)
     }
 
   } else if (PyArg_ParseTuple(args, "O!|sl", &PyFile_Type, &fobject,
-			      &initial, &ibytes)) {
+                              &initial, &ibytes)) {
     PyErr_Clear(); /* clear first failure */
 
+    fname = NULL;
     file = PyFile_AsFile(fobject);
+    if (file) {
+      /* We have to duplicate the file descriptor, since both Python
+         and vorbisfile will want to close it. Don't use the file
+         after you pass it in, or much evil will occur. 
 
-    if (file == NULL) 
+         Really, you shouldn't be passing in files anymore, but in the
+         interest of backwards compatibility it'll stay.
+      */
+      int orig_fd, new_fd;
+      orig_fd = fileno(file);
+      new_fd = dup(orig_fd);
+      file = fdopen(new_fd, "r");
+      if (!file) {
+        PyErr_SetString(PyExc_IOError, "Could not duplicate file.");
+        return NULL;
+      }
+    } else {
       return NULL;
+    }
 
   } else {
     PyErr_SetString(PyExc_TypeError, 
-		    "Argument 1 is not a filename or file object");
+                    "Argument 1 is not a filename or file object");
     return NULL;
   }
 
   self->ovf = (OggVorbis_File*) malloc(sizeof(OggVorbis_File));
+  self->py_file = fobject;
+  Py_XINCREF(fobject); /* Prevent the file from being closed */
   
   retval = ov_open(file, self->ovf, initial, ibytes);
 
-  if (retval < 0)
+  if (retval < 0) {
+    if (fname != NULL)
+      fclose(file);
+    Py_XDECREF(self->py_file);
+
     return v_error_from_code(retval, "Error opening file: ");
+  }
 
   Py_INCREF(Py_None);
   return Py_None;
@@ -230,6 +263,12 @@ py_ov_open(py_vorbisfile *self, PyObject *args)
 
 static char *read_kwlist[] = {"length", "bigendian", "word", "signed", NULL};
 
+static int is_big_endian() {
+  static int x = 0x1;
+  char x_as_char = *(char *) &x;
+  return x_as_char == 0x1 ? 0 : 1;
+}
+
 static PyObject *
 py_ov_read(PyObject *self, PyObject *args, PyObject *kwdict)
 {
@@ -244,13 +283,14 @@ py_ov_read(PyObject *self, PyObject *args, PyObject *kwdict)
   int length, word, sgned, bitstream;
   int bigendianp;
 
-  bigendianp = (__BYTE_ORDER == __BIG_ENDIAN);
+  // Default to host order
+  bigendianp = is_big_endian();
   length = 4096;
   word = 2;
   sgned = 1;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwdict, "|llll", read_kwlist,
-				   &length, &bigendianp, &word, &sgned))
+                                   &length, &bigendianp, &word, &sgned))
     return NULL;
 
   buffobj = PyBuffer_New(length);
@@ -265,14 +305,14 @@ py_ov_read(PyObject *self, PyObject *args, PyObject *kwdict)
   Py_DECREF(tuple);
 
   Py_BEGIN_ALLOW_THREADS
-  retval = ov_read(ov_self->ovf, buff, length, 
-		   bigendianp, word, sgned, &bitstream);
+    retval = ov_read(ov_self->ovf, buff, length, 
+                     bigendianp, word, sgned, &bitstream);
   Py_END_ALLOW_THREADS
 
-  if (retval < 0) {
-    Py_DECREF(buffobj);
-    return v_error_from_code(retval, "Error reading file: ");
-  }
+    if (retval < 0) {
+      Py_DECREF(buffobj);
+      return v_error_from_code(retval, "Error reading file: ");
+    }
 
   retobj = Py_BuildValue("Oii", buffobj, retval, bitstream);
   Py_DECREF(buffobj);
