@@ -37,7 +37,7 @@ char py_vinfo_doc[] =
 Information is stored as attributes.";
 
 PyTypeObject py_vinfo_type = {
-  PyObject_HEAD_INIT(&PyType_Type)
+  PyObject_HEAD_INIT(NULL)
   0,
   "VorbisInfo",
   sizeof(py_vinfo),
@@ -100,7 +100,7 @@ py_info_new(PyObject *self, PyObject *args, PyObject *kwdict)
     return NULL;
   vorbis_info_init(&vi);
 
-  if (quality > 0.0) {
+  if (quality > -1.0) {
     res = vorbis_encode_init_vbr(&vi, channels, rate, quality);
   } else {
     res = vorbis_encode_init(&vi, channels, rate,
@@ -233,7 +233,6 @@ not a list of (string, list) tuples.";
 static void py_vorbis_comment_dealloc(PyObject *);
 static PyObject *py_vorbis_comment_getattr(PyObject *, char *name);
 
-
 static PyMethodDef py_vcomment_methods[] = {
   {"clear", py_vorbis_comment_clear, 
    METH_VARARGS, py_vorbis_comment_clear_doc},
@@ -285,7 +284,7 @@ be two separate tuples in items() for 'Artist' and two strings for 'Artist'\n\
 in value().";
 
 PyTypeObject py_vcomment_type = {
-  PyObject_HEAD_INIT(&PyType_Type)
+  PyObject_HEAD_INIT(NULL)
   0,
   "VorbisComment",
   sizeof(py_vcomment),
@@ -333,14 +332,10 @@ py_comment_new_from_vc(vorbis_comment *vc, PyObject *parent)
   return (PyObject *) newobj;
 }
 
-PyObject *
-py_comment_new(PyObject *self, PyObject *args)
+static PyObject *
+py_comment_new_empty(void)
 {
   py_vcomment *newobj;
-
-  if (!PyArg_ParseTuple(args, ""))
-    return NULL;
-
   newobj = (py_vcomment *) PyObject_NEW(py_vcomment, 
                                         &py_vcomment_type);
   if (!newobj)
@@ -427,6 +422,12 @@ py_comment_subscript(py_vcomment *self,
     int vallen = strlen(res);
 #if PY_UNICODE
     item = PyUnicode_DecodeUTF8(res, vallen, NULL);
+    if (!item) {
+      /* must clear the exception raised by PyUnicode_DecodeUTF8() */
+      PyErr_Clear();
+      /* To deal with non-UTF8 comments (against the standard) */
+      item = PyString_FromStringAndSize(res, vallen); 
+    }
 #else
     item = PyString_FromStringAndSize(res, vallen);
 #endif
@@ -560,9 +561,11 @@ py_comment_items(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, ""))
     return NULL;
 
-  retlist = PyList_New(0);
   dict = py_comment_as_dict(self, NULL);
+  if (!dict)
+    return NULL;
 
+  retlist = PyList_New(0);
   curitem = curpos = 0;
 
   while (PyDict_Next(dict, &curitem, &key, &val) > 0) {
@@ -670,6 +673,29 @@ py_vorbis_comment_query_count(PyObject *self, PyObject *args)
   return PyInt_FromLong(vorbis_comment_query_count(vc, tag));
 }
 
+/* We need this for Win32, so I'll implement it rather than use
+   strcasecmp on Linux */
+static int pystrcasecmp(const char *str1, const char *str2) {
+  int k = 0;
+  while (str1[k] != '\0' && str2[k] != '\0') {
+    char c1 = str1[k];
+    char c2 = str2[k];
+    if (c1 >= 'A' && c1 <= 'Z') {
+      c1 = c1 - 'A' + 'a';
+    }
+    if (c2 >= 'A' && c1 <= 'Z') {
+      c2 = c1 - 'A' + 'a';
+    }
+    if (c1 < c2) {
+      return -1;
+    } else if (c1 > c2) {
+      return 1;
+    }
+    ++k;
+  }
+  return 0;
+}
+
 static int
 make_caps_key(char *in, int size)
 {
@@ -684,6 +710,157 @@ make_caps_key(char *in, int size)
   return 0;
 }
 
+/* Assign a tag in a vorbis_comment, special-casing the VENDOR tag. */
+static int
+assign_tag(vorbis_comment *vcomment, const char *key, PyObject *tag)
+{
+  char *tag_str;
+  char tag_buff[1024];
+  if (PyString_Check(tag)) {
+    tag_str = PyString_AsString(tag);
+  } 
+#if PY_UNICODE
+  else if (PyUnicode_Check(tag)) {
+    tag_str = PyString_AsString(PyUnicode_AsUTF8String(tag));
+  }
+#endif
+  else {
+    PyErr_SetString(PyExc_ValueError, 
+                    "Setting comment with non-string object");
+    return 0;
+  }
+  if (!pystrcasecmp(key, "vendor")) {
+    vcomment->vendor = strdup(tag_str);
+  } else {
+    int k;
+    int key_len = strlen(key);
+    int value_len = strlen(tag_str);
+    if (key_len + value_len + 1 >= sizeof(tag_buff)) {
+      PyErr_SetString(PyExc_ValueError, 
+                      "Comment too long for allocated buffer");
+      return 0;
+    }
+    // Capitalize the key. This is not strictly necessary, but it
+    // keeps things looking consistent.
+    for (k = 0; k < key_len; k++) 
+      tag_buff[k] = toupper(key[k]);
+    tag_buff[key_len] = '=';
+    strncpy(tag_buff + key_len + 1, tag_str, sizeof(tag_buff) - key_len - 1);
+    vorbis_comment_add(vcomment, tag_buff);
+  }
+  return 1;
+}
+
+/* 
+   NOTE:
+   Something like this should be wrong but will, I guess, 'work':
+   { 'Vendor' : 'me', 'VENDOR': 'someone else'} 
+ */
+static int
+create_comment_from_items(vorbis_comment *vcomment, 
+                          const char *key, PyObject *item_vals)
+{
+#if PY_UNICODE 
+  if (PyUnicode_Check(item_vals)) {
+    return assign_tag(vcomment, key, item_vals);
+  } else 
+#endif
+  if (PyString_Check(item_vals)) {
+    return assign_tag(vcomment, key, item_vals);
+  } else if (PySequence_Check(item_vals)) {
+    int j, val_length = PySequence_Length(item_vals);
+    if (!pystrcasecmp(key, "vendor") && val_length > 1) {
+      PyErr_SetString(PyExc_ValueError, "Cannot have multiple vendor tags");
+    }
+    for (j = 0; j < val_length; j++) {
+      PyObject *tag_value = PySequence_GetItem(item_vals, j);
+      if (!tag_value) 
+        return 0;
+      if (!assign_tag(vcomment, key, tag_value))
+        return 0;
+    }
+  } else {
+    PyErr_SetString(PyExc_ValueError, "Value not a string or sequence.");
+    return 0;
+  }
+  return 1;
+}
+
+static vorbis_comment *
+create_comment_from_dict(PyObject *dict)
+{
+  vorbis_comment *vcomment = NULL;
+  int initted = 0;
+  PyObject *items = NULL;
+  int k, length;
+
+  vcomment = (vorbis_comment *) malloc(sizeof(vorbis_comment));
+  if (!vcomment) {
+    PyErr_SetString(PyExc_MemoryError, "error allocating vcomment");
+    goto error;
+  }
+
+  vorbis_comment_init(vcomment);
+  initted = 1;
+  items = PyDict_Items(dict);
+  if (!items)
+    goto error;
+  length = PyList_Size(items);
+  for (k = 0; k < length; k++) {
+    PyObject *pair = PyList_GetItem(items, k);
+    PyObject *key, *val;
+    if (!pair)
+      goto error;
+    assert(PyTuple_Check(pair));
+    key = PyTuple_GetItem(pair, 0);
+    val = PyTuple_GetItem(pair, 1);
+    if (!PyString_Check(key)) {
+      PyErr_SetString(PyExc_ValueError, "Key not a string");
+      goto error;
+    }
+    if (!create_comment_from_items(vcomment, PyString_AsString(key), val))
+      goto error;
+  }
+  
+  return vcomment;
+
+ error:
+  /* Note: I hate dealing with memory */
+  Py_XDECREF(items);
+  if (vcomment) {
+    if (initted)
+      vorbis_comment_clear(vcomment);
+    free(vcomment);
+  }
+  return NULL;
+}
+
+PyObject *
+py_comment_new(PyObject *self, PyObject *args)
+{
+  py_vcomment *pvc;
+  PyObject *dict;
+  vorbis_comment *vcomment;
+  if (PyArg_ParseTuple(args, "")) {
+    return py_comment_new_empty();
+  } else if (!PyArg_ParseTuple(args, "O!", &PyDict_Type, &dict))
+    return NULL;
+  vcomment = create_comment_from_dict(dict);
+  if (!vcomment)
+    return NULL;
+  pvc = (py_vcomment *) PyObject_NEW(py_vcomment,
+                                     &py_vcomment_type);
+  if (!pvc) {
+    vorbis_comment_clear(vcomment);
+    free(vcomment);
+    return NULL;
+  }
+  pvc->vc = vcomment;
+  pvc->parent = NULL;
+  pvc->malloced = 1;
+  return (PyObject *) pvc;
+}
+
 static PyObject *
 py_comment_as_dict(PyObject *self, PyObject *args)
 {
@@ -727,6 +904,10 @@ py_comment_as_dict(PyObject *self, PyObject *args)
       
 #if PY_UNICODE
       item = PyUnicode_DecodeUTF8(val, vallen, NULL);
+      if (!item) {
+        /* To deal with non-UTF8 comments (against the standard) */
+        item = PyString_FromStringAndSize(val, vallen); 
+      } 
 #else
       item = PyString_FromStringAndSize(val, vallen);
 #endif
@@ -785,7 +966,7 @@ write_comments(vorbis_comment *vc, const char *filename, int append)
   strcat(tempfile, ".pytemp");
   
   /* Open the file */
-  in_file = fopen(filename, "r");
+  in_file = fopen(filename, "rb");
   if (!in_file) {
     PyErr_SetFromErrno(PyExc_IOError);
     return NULL;
@@ -845,6 +1026,23 @@ write_comments(vorbis_comment *vc, const char *filename, int append)
   fclose(in_file);
   fclose(out_file);
 
+  /* The following comment, quoted from the vorbiscomment/vcomment.c file
+   * of the vorbis-tools package, regards to, e.g., Windows:
+   * 
+   * Some platforms fail to rename a file if the new name already exists, so
+   * we need to remove, then rename. How stupid.
+   * 
+   * This is why the following block had to be inserted to make things work under
+   * Windows also.
+   * 
+   * <Csaba Henk (ekho@renyi.hu)>
+   */
+  
+  if (remove(filename)) {
+    PyErr_SetFromErrno(PyExc_IOError);
+    return NULL;
+  }
+  
   if (rename(tempfile, filename)) {
     PyErr_SetFromErrno(PyExc_IOError);
     return NULL;
